'  EL_Globals.INC                  TradeStation PowerBASIC DLL Template INC
'  ---------------
'  Version  1.0.0 - 27-June-2003
'
'  Copyright (C) 2003  Roger D. Rines
'
'  emailto:rdrines@spamcop.net?Subject[PB-Bones]
'  =============================================================================
'
'   This library is free software; you can redistribute it and/or
'   modify it under the terms of the GNU Lesser General Public
'   License as published by the Free Software Foundation; either
'   version 2.1 of the License, or (at your option) any later version.
'
'   This library is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'   Lesser General Public License for more details.
'
'   You can read the full GPL Lesser General Public License here:
'   http://www.gnu.org/copyleft/lesser.html
'
'   Or, you should have received a copy of the GNU Lesser General Public
'   License along with this library; if neither happen, you can write to
'   the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
'   Boston, MA  02111-1307  USA
'
'  USE:
'  =============================================================================
'
'  This source file is a PowerBASIC model that is compatible with Omega's
'  TradeStation software.  This code must be used with the PB-Bones.BAS file
'  so the external functions listed here will be meaningful to the DLL.
'
'  PB-Bones.BAS is intended to be a PowerBASIC code-skeleton for creating
'  TradeStation compatible DLL modules.  Nothing in this code is considered final
'  but everything has been tested to see if it would work reliably for the task.
'
'  When this template was published, PowerBASIC's PBWin/DLL version was 7.02.
'  To use this code, you'll probably need to have that version of the
'  PowerBASIC PBWIN/DLL 32-bit compiler to get the same results.
'
'  If you find an error, or know of an enhancement that should be added to this
'  template, your comments will be warmly received and considered for inclusion
'  into this source.  Please send your comments to my email address listed above.
'
'  PURPOSE:
'  =============================================================================
'  This module creates a series of GLOBAL Arrays that can be indexed and used
'  store information.  In this module, there are global elements that can be
'  accessed by all processes and some that are restricted to processes that keep
'  their global index assignment private.
'
'  Because memory is allocated when the DLL is attached to TradeStation, it must
'  be initialized at binding so that memory access violation doesn't occur.
'
'  Later, if a more dynamic memory model is needed, a simple client process can
'  triggered to handle storage and retrieval of global values.
'
'  =============================================================================
'  EXPOSED DEFINITIONS
'  =============================================================================
DECLARE FUNCTION ClearFloat (BYVAL ElementIndex AS LONG) AS LONG 
DECLARE FUNCTION PutFloat (BYVAL FloatValPtr AS DWORD PTR, BYVAL ElementIndex AS LONG) AS LONG 
DECLARE FUNCTION GetFloat (BYVAL ElementIndex AS LONG) AS SINGLE
DECLARE FUNCTION GetString (BYVAL ElementIndex AS LONG) AS STRING 

'  =============================================================================
'  GLOBAL VARIABLE DEFINITIONS
'  =============================================================================

%EL_Globals_Loaded = -1       '  Constant Assignment that communicates this file
                              '  Loaded in the INCLUDE files.
%MaxGlobalElements = 100
%GlobalArrayStart = 0
%ERR_SUBSCRIPT_POINTER_OUT_OF_RANGE = %ERROR_INVALID_BLOCK

TYPE StringRecord
   Text1 AS STRING * 25   
END TYPE               

GLOBAL glStr() AS StringRecord
GLOBAL gFloat() AS SINGLE
     

'  =============================================================================
'     Local Functions and Subs
'  =============================================================================
DECLARE SUB InitializeGlobals 
DECLARE FUNCTION ClearGlobalElement (ElementIndex AS LONG) AS LONG 
        
        
'  =============================================================================
'     Global Handler for PB-BONES.DLL Template
'  =============================================================================

'  -----------------------------------------------------------------------------
'  When the DLL is attached and a DLL_PROCESS_ATTACH message is detected, this
'  SUB must be triggered so the DLL can create enough memory space to handle the
'  global elements requirements.
'
SUB InitializeAllGlobals 
   
   '  Size the Globals
   REDIM glStr(%GlobalArrayStart TO %MaxGlobalElements)
   REDIM gFloat(%GlobalArrayStart TO %MaxGlobalElements)
   
   '  Add whatever else needs to happen here
         
END SUB  '  InitializeGlobals


'  -----------------------------------------------------------------------------
'  This FUNCTION is used to Clear the values from all Global elements assigned 
'  to a process ID Index.  By clearing these elements, they can be used by other  
'  processes that might come along later.  
'
FUNCTION ClearFloat (BYVAL ElementIndex AS LONG) EXPORT AS LONG 

   IF ElementIndex >= %GlobalArrayStart AND ElementIndex <= %MaxGlobalElements THEN
      gFloat(ElementIndex) = 0
      FUNCTION = 0                                 '  Zero Return Indicates no ERROR
   ELSE
      FUNCTION = %ERR_SUBSCRIPT_POINTER_OUT_OF_RANGE   '  Return Error Value 
   END IF

END FUNCTION   '  ClearGlobalElement


'  -----------------------------------------------------------------------------
'  This FUNCTION is used to place a Long value in a Global element assigned 
'  to a process ID Index.  The value assignment is placed in a wrapper FUNCTION  
'  to prevent a program process attempting to assign values to elements not within
'  the GLOBAL array dimension range.  
'
FUNCTION PutFloat (BYVAL FloatValPtr AS DWORD PTR, BYVAL ElementIndex AS LONG) EXPORT AS LONG 
   
   IF ElementIndex >= %GlobalArrayStart AND ElementIndex <= %MaxGlobalElements THEN
      
      gFloat(ElementIndex) = @FloatValPtr
      FUNCTION = 0                                 '  Zero Return Indicates no ERROR
   ELSE
      FUNCTION = %ERR_SUBSCRIPT_POINTER_OUT_OF_RANGE '  Return Error Value 
   END IF

END FUNCTION   '  ClearGlobalElement


'  -----------------------------------------------------------------------------
'  This FUNCTION is used to retrieve a values from a Global element assigned 
'  to a process ID Index.  By using a wrapper FUNCTION, a dimension range test
'  can be performed so an out of bounds error isn't created should the calling
'  program request an element index location not within the Global array range.  
'
FUNCTION GetFloat (BYVAL ElementIndex AS LONG) EXPORT AS SINGLE 
   
   IF ElementIndex >= %GlobalArrayStart AND ElementIndex <= %MaxGlobalElements THEN
      FUNCTION = gFloat(ElementIndex)      
   ELSE
      FUNCTION = 0                     '  Zero Return Indicates possible ERROR 
   END IF

END FUNCTION   '  ClearGlobalElement

'============================================
'============================================
'  -----------------------------------------------------------------------------
'  This FUNCTION is used to Clear the values from all Global elements assigned 
'  to a process ID Index.  By clearing these elements, they can be used by other  
'  processes that might come along later.  
'
'FUNCTION ClearFloat (BYVAL ElementIndex AS LONG) EXPORT AS LONG 
'
'   IF ElementIndex >= %GlobalArrayStart AND ElementIndex <= %MaxGlobalElements THEN
'      gFloat(ElementIndex) = 0
'      FUNCTION = 0                                 '  Zero Return Indicates no ERROR
'   ELSE
'      FUNCTION = %ERR_SUBSCRIPT_POINTER_OUT_OF_RANGE   '  Return Error Value 
'   END IF
'
'END FUNCTION   '  ClearGlobalElement


'  -----------------------------------------------------------------------------
'  This FUNCTION is used to place a Long value in a Global element assigned 
'  to a process ID Index.  The value assignment is placed in a wrapper FUNCTION  
'  to prevent a program process attempting to assign values to elements not within
'  the GLOBAL array dimension range.  
'
FUNCTION PutString (BYVAL FloatValPtr AS DWORD PTR, BYVAL ElementIndex AS LONG) EXPORT AS LONG 
   
   IF ElementIndex >= %GlobalArrayStart AND ElementIndex <= %MaxGlobalElements THEN
      
      gFloat(ElementIndex) = @FloatValPtr
      FUNCTION = 0                                 '  Zero Return Indicates no ERROR
   ELSE
      FUNCTION = %ERR_SUBSCRIPT_POINTER_OUT_OF_RANGE   '  Return Error Value 
   END IF

END FUNCTION   '  ClearGlobalElement


'  -----------------------------------------------------------------------------
'  This FUNCTION is used to retrieve a values from a Global element assigned 
'  to a process ID Index.  By using a wrapper FUNCTION, a dimension range test
'  can be performed so an out of bounds error isn't created should the calling
'  program request an element index location not within the Global array range.  
'
FUNCTION GetString (BYVAL ElementIndex AS LONG) EXPORT AS STRING 
   
   IF ElementIndex >= %GlobalArrayStart AND ElementIndex <= %MaxGlobalElements THEN
      FUNCTION = glStr(ElementIndex)      
   ELSE
      FUNCTION = ""                     '  Zero Return Indicates possible ERROR 
   END IF

END FUNCTION   '  ClearGlobalElement
'  =============================================================================
'     END OF CODE for EL_Globals.INC
'  =============================================================================
