'  =============================================================================
'  PB-Add-Free.INC                   TradeStation PowerBASIC DLL Template INC
'  ---------------
'  Version  1.0.0 - 27-June-2003
'
'  Copyright (C) 2003  Roger D. Rines
'
'  emailto:rdrines@spamcop.net?Subject[PB-Bones]
'  =============================================================================
'
'   This library is free software; you can redistribute it and/or
'   modify it under the terms of the GNU Lesser General Public
'   License as published by the Free Software Foundation; either
'   version 2.1 of the License, or (at your option) any later version.
'
'   This library is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'   Lesser General Public License for more details.
'
'   You can read the full GPL Lesser General Public License here:
'   http://www.gnu.org/copyleft/lesser.html
'
'   Or, you should have received a copy of the GNU Lesser General Public
'   License along with this library; if neither happen, you can write to
'   the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
'   Boston, MA  02111-1307  USA
'
'  USE:
'  =============================================================================
'
'  This source file is a PowerBASIC model that is compatible with Omega's
'  TradeStation software.  This code must be used with the PB-Bones.BAS file
'  so the external functions listed here will be meaningful to the DLL.
'
'  PB-Bones.BAS is intended to be a PowerBASIC code-skeleton for creating
'  TradeStation compatible DLL modules.  Nothing in this code is considered final
'  but everything has been tested to see if it would work reliably for the task.
'
'  When this template was published, PowerBASIC's PBWin/DLL version was 7.02.
'  To use this code, you'll probably need to have that version of the
'  PowerBASIC PBWIN/DLL 32-bit compiler to get the same results.
'
'  If you find an error, or know of an enhancement that should be added to this
'  template, your comments will be warmly received and considered for inclusion
'  into this source.  Please send your comments to my email address listed above.
'


'  =============================================================================
'  EXPOSED DEFINITIONS
'  =============================================================================
 
Declare Sub Dll_Add Lib "PB-BONES.DLL" Alias "Dll_Add" (ByVal elThreadId As Dword)
Declare Sub Dll_Free Lib "PB-BONES.DLL" Alias "Dll_Free" (ByVal elThreadId As Dword)
Declare Sub Dll_Context Lib "PB-BONES.DLL" Alias "Dll_Context" (ByVal elThreadId As Dword)

DECLARE FUNCTION GetIDNumIndex(BYVAL ELIdNum AS LONG) AS LONG


'  =============================================================================
'  GLOBAL VARIABLE DEFINITIONS
'  =============================================================================

%PB_Add_Free = -1             '  Constant Assignment that communicates this file
                              '  Loaded in the INCLUDE files.

%MaxScreenNums = 10           '  Arbitrary value to limit the Maximum number of
                              '  processes that can reliably attach to this program.
                              

TYPE EL_ProcessRecord         '  UDT Record to manage process ID values
   hID AS LONG                '  ID Value submitted to DLL_ADD
   hIndex AS LONG             '  Element Location Index reference
   hUsed AS BYTE              '  FLAG to show previous use
END TYPE


'  Record Array to hold the Process ID Number and its Index location.
GLOBAL TsID () AS EL_ProcessRecord '  Keep ZERO Position for Error Location

GLOBAL elID AS DWORD          '  Holds current EL-Instance Handle

'  Counts of the # of times an EL-Instance calls one of the Thread routines - Debug
GLOBAL AddCount&, FreeCount&, ContextCount&


GLOBAL IdIndex AS LONG
GLOBAL WorkID AS LONG
GLOBAL LastAddID AS LONG, LastContextID AS LONG, LastFreeID AS LONG
GLOBAL LogMsg$

'  =============================================================================
'     Local Functions and Subs
'  =============================================================================

DECLARE FUNCTION StoreELID( ELIDNum AS LONG) AS LONG
DECLARE FUNCTION GetNextIndex( CurrentIndex AS LONG) AS LONG
DECLARE FUNCTION RemoveIDNum(ELIdNum AS LONG) AS LONG
DECLARE FUNCTION FindIDNum(ELIDNum AS LONG) AS LONG
DECLARE SUB InitIdRecords


'  =============================================================================
'     Dll_Add, Dll_Context & Dll_Free  - Call by Easy Language hidden processes
'  =============================================================================
'
'  If these SUB - Routines are not exposed or included in a DLL, Easy Language
'  ignores them, but the programmer looses the ability to syncronize and direct
'  calls from the various workspaces because the calling ID will be lost.

'  ....................
'  Used to initialize, a thread space, mutex or semiphore
'  This SUB is called when the program that uses this DLL becomes active in the
'  chart space.
'
Sub Dll_Add(ByVal elThreadId As Dword) Export   '  Establish Space for ID#

   elID = elThreadId
   INCR AddCount&

   IF ElID = LastAddID THEN
      '   Don't Change the Index value or look it up

      LogMsg$ = "Dll_Add - CallCount = " +_
               STR$(AddCount&) +_
               "  elID =" + STR$(elThreadId)+_
               " Ignore This Call"                                '  - Debug

   ELSE
      '  This ID Number is different, check it out for storage action
      LastAddID = ElID                         '  Keep an ID copy
      IDIndex = StoreELID( ElID)               '  Store Current ID Index

      LogMsg$ = "Dll_Add - CallCount = " + STR$(AddCount&) +_
            "  elID =" + STR$(elThreadId) +_
             " ID Index=" + STR$(IDIndex)                         '  - Debug
   END IF   '  ElID = LastAddID

   CALL WriteLog( LogMsg$)                                        '  - Debug
END SUB  '  Dll_Add

'  ....................
'  Used to direct calls to a thread space, mutex or semiphore
'  This SUB is called each time a program wants information from the DLL, which in
'  many, if not most, case is once for each bar loaded into the chart space.
'  In other words, this SUB gets called a lot of time for each chart space.
'
Sub Dll_Context(ByVal elThreadId As Dword) Export  '   Direct work to space ID#

   elID = elThreadId
   INCR ContextCount&

   IF ElID = LastAddID THEN
     '   Don't Change the Index value or look it up

      LogMsg$ = "Dll_Context - ID No Change!"    '  - Debug

   ELSE
      '  This ID Number is different, check it out for storage action
      LastAddID = ElID                         '  Keep an ID copy
      IDIndex = StoreELID( ElID)               '  Store Current ID Index

      LogMsg$ = "Dll_Context - CallCount ="_
               + STR$(ContextCount&) +_
               "  elID =" + STR$(elThreadId) +_
               " ID Index=" + STR$(IDIndex)                       '  - Debug
   END IF   '  ElID = LastAddID

   CALL WriteLog( LogMsg$)                                        '  - Debug
END SUB  '  Dll_Context

'  ....................
'  Used to terminate a thread space, mutex or semiphore
'  For each time Dll_Add is called, a new ID# is issued.  For each ID# issued,
'  this SUB is called when the status of the programs are turned off, or the chart
'  is closed.  There will always be an equal number of calls to this SUB as there are
'  the Dll_Add SUB.
'
Sub Dll_Free(ByVal elThreadId As Dword) Export  '  Terminate Protected Space for an ID#
   DIM hIndex AS LONG

   elID = elThreadId
   INCR FreeCount&

   '  Remove this ID Number from the place holder list
   hIndex = RemoveIDNum(elID)

   LogMsg$ = "Dll_Free - CallCount = " + STR$(FreeCount&) +_      '  - Debug
          "  elID =" + STR$(elThreadId)+ " hIndex=" + STR$(hIndex)'  - Debug
   CALL WriteLog( LogMsg$)                                        '  - Debug
END SUB  '  Dll_Free



'  =============================================================================
'                 ID Handling Routines
'  =============================================================================

'  -----------------------------------------------------------------------------
'  This Function is called by DLL_ADD and returns ZERO if it is successful

FUNCTION StoreELID( ELIDNum AS LONG) AS LONG
   DIM IDTest AS LONG

   '  Check to be sure ID number hasn't been given before - Error Check
   IDTest = FindIDNum(ELIDNum)

   IF IDTest = 0 THEN   '  Number wasn't found in the list

      '  Look for next available Index location
      IdIndex = GetNextIndex(IdIndex)

      '  Store the TradeStation ID number in an element of the array
      TSID(IdIndex).hID = ELIDNum

      '  Turn In-Use FLAG ON
      TSID(IdIndex).hUsed = 1

      FUNCTION = IdIndex
   ELSE
      ' If you get here, you are in trouble
'      MSGBOX "This number:" + STR$(ELIDNum) + " was used before!"        ' - Debug

      '  When we get here, we either panic or get creative in how to handle the
      '  ID duplication.  A way to handle it is to clear the variables for working array
      '  data array assigned to that ID series.

      '  I don't know if a screen is closed whether EL will re-use ID numbers.  I haven't
      '  seen it with my testing, but that hasn't been exhaustive

      '  If they do re-use them, then removing them when the EL-screen is closed is best
      FUNCTION = IDTest
   END IF

END FUNCTION   '  StoreELID

'  -----------------------------------------------------------------------------
'  This Function locates the next available Index location
'
FUNCTION GetNextIndex(CurrentIndex AS LONG) AS LONG
   DIM TempIndex AS LONG
   DIM ElNum AS LONG
   DIM NextID AS LONG
   DIM LoopCount AS LONG

   IF CurrentIndex <= %MaxScreenNums AND CurrentIndex > 0 THEN '  Are we below the Processing Limit?
      TempIndex = CurrentIndex            '  If yes, begin looking here
   ELSE
      TempIndex = 1                       '  Start at the beginning hoping for a hole
   END IF   '  CurrentIndex <= %MaxScreenNums

   LoopCount = 0
   NextID = 0

   '  Search Index Array for next available Index
   FOR ElNum = TempIndex TO %MaxScreenNums

      IF TSID(ElNum).hID = 0 THEN         '  If we find an open Element,...
        NextID = TSID(ElNum).hIndex       '  Use the Index Value associated with it
        EXIT FOR
      ELSE
         INCR LoopCount
      END IF   '  TSID(ElNum).hID = 0

   NEXT ElNum

   IF NextID = 0 AND LoopCount < %MaxScreenNums THEN

      FOR ElNum = 1 TO %MaxScreenNums

         IF TSID(ElNum).hID = 0 THEN
           NextID = TSID(ElNum).hIndex
           EXIT FOR
         END IF
      NEXT ElNum

      NextID = 0
   END IF

   FUNCTION = NextID

END FUNCTION   '  GetNextIndex


'  -----------------------------------------------------------------------------
'  This function is called by DLL_CONTEXT and it returns the Index value
'  assigned to the referenced ID number
'
FUNCTION GetIDNumIndex(BYVAL ELIdNum AS LONG) EXPORT AS LONG
   
   '  Locate Index for this ID Number
   IdIndex = FindIDNum(ELIDNum)

   FUNCTION = IdIndex   '  If ZERO, ID Number isn't in ID-List

END FUNCTION   '  GetIDNumIndex


'  -----------------------------------------------------------------------------
'  This function, called by DLL_FREE, removes ID numbers from the ARRAY by
'  setting TSID(IDTest).hID = 0.
'
'  It preserves the Index Element as a place holder for the next process that
'  needs its number stored, and it clears the IN-Use FLAG after it Initializes
'  the Global variables.
'
FUNCTION RemoveIDNum(ELIdNum AS LONG) AS LONG

   '  Get the Element position for this ID number
   IdIndex = FindIDNum(ELIDNum)

   '  Clear the ID number from the place holder
   TSID(IdIndex).hID = 0

   IF TSID(IdIndex).hUsed = 1 THEN  '  Should always be true

      '  Initialize Global Variables to Remove Old Data
      ' Call a Clearing Function HERE

      '  Turn Off Contamination FLAG
      TSID(IdIndex).hUsed = 0
   END IF

   FUNCTION = IdIndex    '   Return the Element number position - ZERO indicates ERROR
END FUNCTION   '  RemoveIDNum


'  -----------------------------------------------------------------------------
'  This function does a linear search of the small array by looking at all the
'  elements for a matching ID value.
'
FUNCTION FindIDNum(ELIDNum AS LONG) AS LONG
   DIM IDTest AS LONG
   DIM ElNum AS LONG

   IDTest = 0

   FOR ElNum = 1 TO UBOUND(TSID)

      '  Look for the first empty element location
      IF TSID(ElNum).hID = ELIDNum THEN
         IDTest = TSID(ElNum).hIndex   '  This Index value is available

         EXIT FOR
      END IF   '  TSID(ElNum).hID = 0
   NEXT ElNum

   FUNCTION = IDTest                   '  If ZERO is returned, list is full

END FUNCTION   '  FindIDNum


'  -----------------------------------------------------------------------------
'  When PB-Add-Free.INC is loaded into a DLL-Template, %DLL_PROCESS_ATTACH calls
'  this SUB to Initialize the INDEX and Clear all the Records.  If this module
'  isn't present, %DLL_PROCESS_ATTACH ignores procedure
'
SUB InitIdRecords
   DIM ELNum AS LONG, ErrNum AS LONG
   DIM ErrorPlace AS STRING

   '  Size the Array here
   REDIM TsID(%MaxScreenNums)

   Msg$ = "InitIdRecords - ElNum = " + Str$(ElNum)           ' - Debug
   Call WriteLog( Msg$)                            '  Write Debug Message - Debug

   '  Clear all ID Elements and Insert Index Values
   FOR ElNum = 1 TO %MaxScreenNums
      TSID(ElNum).hID = 0
      TSID(ElNum).hIndex = ElNum
      TSID(ElNum).hUsed = 0
   NEXT ElNum
   
   IF ErrNum <> 0 THEN
      ErrorPlace = "Error #:" + STR$(ERR) + " occurred in " + FUNCNAME$
      CALL SendFastMsg("All Globals Initialized", ErrorPlace )
   END IF
   
END SUB  '  InitIdRecords

'  =============================================================================
'                 ID Handling Routines
'  =============================================================================
