'  =============================================================================
'  PB-Bones.BAS                    TradeStation PowerBASIC DLL Template
'  ------------
'  Version  1.0.0 - 27-June-2003
'
'  Copyright (C) 2003  Roger D. Rines
'
'  emailto:rdrines@spamcop.net?Subject[PB-Bones]
'  =============================================================================
'   
'   This library is free software; you can redistribute it and/or
'   modify it under the terms of the GNU Lesser General Public
'   License as published by the Free Software Foundation; either
'   version 2.1 of the License, or (at your option) any later version.
'   
'   This library is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'   Lesser General Public License for more details.
'   
'   You can read the full GPL Lesser General Public License here:
'   http://www.gnu.org/copyleft/lesser.html
'   
'   Or, you should have received a copy of the GNU Lesser General Public
'   License along with this library; if neither happen, you can write to 
'   the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
'   Boston, MA  02111-1307  USA
'   
'  USE:
'  =============================================================================
'  
'  This source file is a PowerBASIC model that is compatible with Omega's 
'  TradeStation software.  This code must be used with the ELKit32PB.INC file 
'  so the external functions of TradeStation's ELKit32.DLL module are accessible 
'  to this DLL.
'
'  PB-Bones.BAS is intended to be a PowerBASIC code-skeleton for creating 
'  TradeStation compatible DLL modules.  Nothing in this code is considered final
'  but everything has been tested to see if it would work reliably for the task.
'
'  Error prevention is not included as part of the skeleton because that chore 
'  is usually incorporated as a global process, or as part of the code logic.  
'  While I use error checking to handle potential problems, many people don't 
'  and don't seem to have much trouble.  No matter what you do your GPF points 
'  will vary.
'
'  When this template was published, PowerBASIC's PBWin/DLL version was 7.02.
'  To use this code, you'll probably need to have that version of the  
'  PowerBASIC PBWIN/DLL 32-bit compiler to get the same results.
'
'  If you find an error, or know of an enhancement that should be added to this
'  template, your comments will be warmly received and considered for inclusion
'  into this source.  Please send your comments to my email address listed above.
'
'  =============================================================================
    
#DIM ALL

#COMPILE DLL "PB-Bones.DLL"      '  Prefix part can be almost any name you like
'#INCLUDE "WIN32API.INC"         '  Everything in WIN32API isn't needed in Template,
                                 '  so PB-Bones.INC is being used in this example

#INCLUDE "PB-Bones.INC"          '  This INC contains the minimum required Equates 

#Include "PB-DebugLog.INC"       '  Contains Debug LogFile Functions

#Include "PB-Add-Free.INC"       '  Includes the DLL_ADD, DLL_CPNTEXT & DLL_FREE 
                                 '  SUB calls TradeStation will use when they are
                                 '  included in a DLL.                  
                                 
#INCLUDE "EL_Globals.INC"        '  Routines for handling the storage and retrieval
                                 '  of Global values from array elements
                                 
#INCLUDE "ELKit32PB.INC"         '  TradeStation API declaration file  



'  =============================================================================
'  GLOBAL VARIABLE DEFINITIONS
'  =============================================================================
                              '  Elements that show "Debug" can be removed 
                              '  as they were only inserted to give a new user 
                              '  an idea of what happens when the DLL is 
                              '  loaded and accessed.
                              
GLOBAL gNumOfTimes AS DWORD   '  Counts # of times LibMain is called - Debug


'  =============================================================================
'  EQUATE DEFINITIONS
'  =============================================================================
$DLL_VERSION = "1.0.0"        '  Holds the Current DLL version Number
                                                                                

#IF NOT %DEF(%PB_Add_Free)    '  If PB-Add_Free.INC isn't used, set to ZERO
   %PB_Add_Free = -1
#ENDIF                  
                                                                                
'  =============================================================================                                                                                
'  EXPOSED DEFINITIONS
'  =============================================================================
   
DECLARE FUNCTION OpenPriceVal (BYVAL OpenPtr AS DWORD,_               '  Pointer to Variable
                               BYVAL OffSet AS LONG,_                 '  Bar Offset Value
                               BYVAL Scale AS SINGLE) AS SINGLE       '  PriceScale Value
                               
DECLARE FUNCTION HighPriceVal (BYVAL HighPtr AS DWORD,_               '  Pointer to Variable
                               BYVAL OffSet AS LONG,_                 '  Bar Offset Value
                               BYVAL Scale AS SINGLE) AS SINGLE       '  PriceScale Value
                               
DECLARE FUNCTION LowPriceVal (BYVAL LowPtr AS DWORD,_                 '  Pointer to Variable
                              BYVAL OffSet AS LONG,_                  '  Bar Offset Value
                              BYVAL Scale AS SINGLE) AS SINGLE        '  PriceScale Value
                              
DECLARE FUNCTION ClosePriceVal (BYVAL ClosePtr AS DWORD,_             '  Pointer to Variable
                                BYVAL OffSet AS LONG,_                '  Bar Offset Value
                                BYVAL Scale AS SINGLE) AS SINGLE      '  PriceScale Value

DECLARE FUNCTION VolumeVAL (BYVAL VolPTR AS DWORD,_                   '  Pointer to Variable
                            BYVAL OffSet AS LONG) AS SINGLE           '  Bar Offset Value
                            
DECLARE FUNCTION OpenIntVAL (BYVAL OpnIntPTR AS DWORD,_               '  Pointer to Variable
                             BYVAL OffSet AS LONG) AS SINGLE          '  Bar Offset Value

DECLARE FUNCTION UpTickVAL (BYVAL UpTikPTR AS DWORD,_                 '  Pointer to Variable
                            BYVAL OffSet AS LONG) AS SINGLE           '  Bar Offset Value
                            
DECLARE FUNCTION DownTickVAL (BYVAL DnTikPTR AS DWORD,_               '  Pointer to Variable
                              BYVAL OffSet AS LONG) AS SINGLE         '  Bar Offset Value

DECLARE FUNCTION BarDateJulianVAL (BYVAL BarDatePTR AS DWORD,_        '  Pointer to Variable
                                   BYVAL OffSet AS LONG) AS SINGLE    '  Bar Offset Value
                                   
DECLARE FUNCTION BarDateYMD(BYVAL BarDatePTR AS DWORD,_               '  Pointer to Variable
                            BYVAL OffSet AS LONG) AS SINGLE           '  Bar Offset Value

DECLARE FUNCTION BarTimeMinutes (BYVAL BarTimePTR AS DWORD,_          '  Pointer to Variable
                                 BYVAL OffSet AS LONG) AS SINGLE      '  Bar Offset Value
                                 
DECLARE FUNCTION BarTimeHHMM(BYVAL BarTimePTR AS DWORD,_              '  Pointer to Variable
                             BYVAL OffSet AS LONG) AS SINGLE          '  Bar Offset Value

DECLARE FUNCTION VariableVAL (BYVAL VariablePTR AS DWORD,_            '  Pointer to Variable
                              BYVAL OffSet AS LONG,_                  '  Bar Offset Value
                              BYVAL VarStartAddr AS DWORD,_           '  Variable Starting Address
                              BYVAL VarSize AS LONG) AS SINGLE        '  Variable Type Size

DECLARE FUNCTION ArrayElementValue (BYVAL ArrayElement AS DWORD,_     '  Pointer to Array-Element
                                    BYVAL OffSet AS LONG,_            '  Bar Offset Value
                                    BYVAL StartAddr AS DWORD,_        '  Start Address of ARRAY
                                    BYVAL ArraySize AS DWORD) AS SINGLE' ArraySize Value
	
DECLARE FUNCTION dllAverageClose(BYVAL PricePTR AS DWORD,_            '  Pointer to Variable
                                 BYVAL OffSet AS LONG,_               '  Bar Offset Value
                                 BYVAL Length AS LONG,_               '  Look back Value
                                 BYVAL Scale AS SINGLE) AS SINGLE     '  PriceScale Value

DECLARE FUNCTION MinutesToTime (BYVAL nMinutes AS SINGLE) AS SINGLE   '  Converts Minutes to HHMM
DECLARE FUNCTION TimeToMinutes (BYVAL ClockTime AS SINGLE) AS SINGLE  '  Won't validate Time received

DECLARE FUNCTION DoubleIt (BYVAL AnyPrice AS SINGLE) AS SINGLE        ' Simple Value Doubler
DECLARE FUNCTION IncrNum (BYVAL AnyVariableNum AS SINGLE) AS SINGLE   ' Increments a Number by 1

'DECLARE FUNCTION MakeCalDate (BYVAL ELDateStr AS ASCIIZ PTR,_         '  Pass Date as STRING PTR
'                              BYVAL NotUS AS LONG) AS STRING          '  Returns a Modified STRING    

DECLARE FUNCTION MakeCalDate (ELDateStr AS ASCIIZ,_         '  Pass Date as STRING PTR
                              BYVAL NotUS AS LONG) AS STRING          '  Returns a Modified STRING    

'  =============================================================================
'  LOCAL DEFINITIONS - NOT Exposed
'  =============================================================================
'  Functions and Subs to show functionality - Not needed unless you want them
'  =============================================================================

                                                                    

'  =============================================================================
'                          ACCESS MODULES START HERE
'  =============================================================================

'  Main DLL Module Function is called when the DLL is attached and when it 
'  is terminated.  If any initialization of the DLL is needed, create a procedure 
'  that can be called when '%DLL_PROCESS_ATTACH' is triggered.
' 
'  When the DLL is ready to terminate, put a call to cleanup procedures
'  in the '%DLL_THREAD_DETACH' section.  
'
'  When an Easy Language Program is attached to the DLL, TradeStation will trigger 
'  an event that will call DLL_ADD and a ID-number will be passed to the DLL.  
'
'  When TradeStation detaches from the DLL, it will trigger an event that calls
'  the DLL_FREE procedure, and it will tell the DLL which instance is terminating.  
'
'  All program request for information from the DLL will trigger a call to 
'  DLL_CONTEXT procedure and it will include the instance ID-number originally
'  established by the EL program running in the space of the identified instance.
'
'  Each time Easy Language attaches a DLL to a program, it sends the DLL an event
'  handle that can be used to identify the process that is attaching, requesting 
'  and detaching.  This event-handle identification passed with each call 
'  will allow the programmer of the DLL to create seperate THREADS, or change 
'  the current index value on global arrays that aren't intended to be accessed
'  across instances.
'
'  IF the SUBs DLL_ADD, DLL_CONTEXT & DLL_FREE are left out of the code or not
'  exposed, TradeStation will detect their absence and won't issue any calls to
'  them.  It also won't be sending any ID information as well, so if knowing 
'  which process is asking the DLL for something isn't important, leave the 
'  SUBs out and a tiny speed improvement may be the result.
'
'  EL = Easy Language
'  =============================================================================
'  LIBMAIN is always called when it is attached to another program.  When the first
'  Easy Language program sends the '%DLL_PROCESS_ATTACH' call, the processing and 
'  communication environment is established and '%DLL_PROCESS_ATTACH' is not called 
'  again until it has been released by all processes accessing the DLL.  This means,
'  that as long as any Easy Language program is using the '%DLL_THREAD_DETACH' 
'  signal will not be sent.
'
FUNCTION LIBMAIN(BYVAL hInstance AS DWORD,_
                 BYVAL Reason    AS LONG,_
                 BYVAL Reserved  AS LONG) AS LONG
   
   DIM Msg$                                              '  Variable for Debug - Debug
   
   INCR gNumOfTimes                                      '  Counts LibMain calls - Debug
   
   SELECT CASE Reason
      CASE %DLL_PROCESS_ATTACH
         '  Start the Logging process
         LogNum% = StartLog%("C:\Z\TS-DLL\CallLog.TXT")  '  Open named log file - Debug

         '  Write a Log Record
         Msg$ = "%Dll_PROCESS_ATTACH - " + _
                "%PB_Add_Free = " + Str$(%PB_Add_Free)                        ' - Debug
         Call WriteLog( Msg$)                            '  Write Debug Message - Debug

         '  Write a Log Record
         Msg$ = "%Dll_PROCESS_ATTACH - Count = " + " elID = " +_
               Str$(elID)+ " Lib Call Count = " + Str$(gNumOfTimes)           ' - Debug
         Call WriteLog( Msg$)                            '  Write Debug Message - Debug

         
         '   initialize DLL here with whatever you need initialized        
         IF %PB_Add_Free THEN InitIdRecords              '  Initializes ID Records         
         
         '  Write a Log Record
         Msg$ = "%Dll_PROCESS_ATTACH - Count = " + " elID = " +_
               Str$(elID)+ " Lib Call Count = " + Str$(gNumOfTimes)           ' - Debug
         Call WriteLog( Msg$)                            '  Write Debug Message - Debug
         
         LIBMAIN = 1
         EXIT FUNCTION                                   '  LIBMAIN


      CASE %DLL_PROCESS_DETACH
         '   This is where app unloads the DLL
         
         '  END the Logging process
         Msg$ = "%Dll_PROCESS_DETACH - Count = " + " elID = " +_
                STR$(elID)+ " Lib Call Count = " + STR$(gNumOfTimes)          ' - Debug
         CALL WriteLog( Msg$)                            '  Write Debug Message - Debug
         CALL EndLog                                     '  CLOSE this named log file - Debug
         
         EXIT FUNCTION                                   '  LIBMAIN
         
         
      CASE %DLL_THREAD_ATTACH
         '   This is like %DLL_PROCESS_ATTACH
         Msg$ = "%Dll_THREAD_ATTACH - Count = " + " elID = " +_
                STR$(elID)+ " Lib Call Count = " + STR$(gNumOfTimes)          ' - Debug
         CALL WriteLog( Msg$)                            '  Write Debug Message - Debug
         
         EXIT FUNCTION                                   '  LIBMAIN
   
   
      CASE %DLL_THREAD_DETACH                                
         '   This is like %DLL_PROCESS_DETACH
         
         '   Global Variables, etc. will be lost at this point
         Msg$ = "%Dll_THREAD_DETACH - Count = " + " elID = " +_
                 STR$(elID)+ " Lib Call Count = " + STR$(gNumOfTimes)         ' - Debug
         CALL WriteLog( Msg$)                            '  Write Debug Message - Debug
         
         EXIT FUNCTION                                   '  LIBMAIN
      
      END SELECT  '  CASE Reason
   
   
   ' If you get here, you are in serious trouble
   MSGBOX "Windows Error in PB-Dones.DLL - LIBMAIN() - Reboot Windows"        ' - Debug
   
   '   A Serious Failure has Occured!
   LIBMAIN = 0  
END FUNCTION   '  LIBMAIN


'  =============================================================================
'                          EXPORTED FUNCTIONS & SUBroutines
'  =============================================================================
'
'  These address access FUNCTIONSs are the calls EL, or other SUBs and FUNCTIONS
'  in the DLL can use to simplify access to the data ojects used by TradeStation.
'
'  ------------------------------------
'  Finds the Price Value for the OPEN at Offset Location
FUNCTION OpenPriceVal(BYVAL OpenPtr AS DWORD, BYVAL OffSet AS LONG,_
                      BYVAL Scale AS SINGLE) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER

   '  Get the Address of this Price Bar Element
   ElementPTR = FindAddress_Open ( OpenPtr, OffSet)

   ' Adjust Returned Value to Reflect Price Storage Adjustments
   IF Scale <> 0 THEN
      FUNCTION = @ElementPTR / Scale
   ELSE
      FUNCTION = @ElementPTR
   END IF   '  Scale <> 0 

END FUNCTION   '  OpenPriceVal

'  ------------------------------------
'  Finds the Price Value for the HIGH at Offset Location
FUNCTION HighPriceVal(BYVAL HighPtr AS DWORD, BYVAL OffSet AS LONG,_
                      BYVAL Scale AS SINGLE) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER
   
   '  Get the Address of this Price Bar Element
   ElementPTR = FindAddress_High( HighPtr, OffSet)
   
   ' Adjust Returned Value to Reflect Price Storage Adjustments
   IF Scale <> 0 THEN
      FUNCTION = @ElementPTR / Scale
   ELSE
      FUNCTION = @ElementPTR
   END IF   '  Scale <> 0 

END FUNCTION   '  HighPriceVal

'  ------------------------------------
'  Finds the Price Value for the LOW at Offset Location
FUNCTION LowPriceVal(BYVAL LowPtr AS DWORD, BYVAL OffSet AS LONG,_
                     BYVAL Scale AS SINGLE) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER
   
   '  Get the Address of this Price Bar Element
   ElementPTR = FindAddress_Low( LowPtr, OffSet)
   
   ' Adjust Returned Value to Reflect Price Storage Adjustments
   IF Scale <> 0 THEN
      FUNCTION = @ElementPTR / Scale
   ELSE
      FUNCTION = @ElementPTR
   END IF   '  Scale <> 0 

END FUNCTION   '  LowPriceVal

'  ------------------------------------
'  Finds the Price Value for the CLOSE at Offset Location
FUNCTION ClosePriceVal (BYVAL ClosePtr AS DWORD, BYVAL OffSet AS LONG,_
                        BYVAL Scale AS SINGLE) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER
           
   '  Get the Address of this Price Bar Element
   ElementPTR = FindAddress_Close( ClosePTR, OffSet)        
      
   ' Adjust Returned Value to Reflect Price Storage Adjustments
   IF Scale <> 0 THEN
      FUNCTION = @ElementPTR / Scale   '  Scale = Chart display setting
   ELSE
      FUNCTION = @ElementPTR
   END IF   '  Scale <> 0 

END FUNCTION   '  ClosePriceVal

'  ------------------------------------
'  Finds the Price Value for the VOLUME at Offset Location
FUNCTION VolumeVAL(BYVAL VolPTR AS DWORD, BYVAL OffSet AS LONG) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER   

   '  Get the Address of this Price Bar Element	   
   ElementPTR = FindAddress_Volume( VolPTR, OffSet)	   
'LPLONG FAR PASCAL FindAddress_Volume (LPLONG lpVolume, INT nOfs);

   FUNCTION = @ElementPTR                                             
  
END FUNCTION   '  VolumeVAL

'  ------------------------------------
'  Finds the Price Value for the OPEN INTEREST at Offset Location
FUNCTION OpenIntVAL(BYVAL OpnIntPTR AS DWORD, BYVAL OffSet AS LONG) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER   
   
   ElementPTR = FindAddress_OpenInt(OpnIntPTR, OffSet)
	
   FUNCTION = @ElementPTR                                             
  
END FUNCTION   '  OpenIntVAL

'  ------------------------------------
'  Finds the Price Value for the UpTick or VOLUME at Offset Location
FUNCTION UpTickVAL(BYVAL UpTikPTR AS DWORD,_
                   BYVAL OffSet AS LONG) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER   
   
   ElementPTR = FindAddress_UpTicks( UpTikPTR, OffSet)

   FUNCTION = @ElementPTR                                             
  
END FUNCTION   '  UpTickVAL

'  ------------------------------------
'  Finds the Price Value for the DownTick or OPEN INTERST at Offset Location
FUNCTION DownTickVAL(BYVAL DnTikPTR AS DWORD,_
                     BYVAL OffSet AS LONG) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER   
   
   ElementPTR = FindAddress_DownTicks( DnTikPTR, OffSet)

   FUNCTION = @ElementPTR                                             
  
END FUNCTION   '  DownTickVAL

'  ------------------------------------
'  Returns the Value of an Easy Language Variable at OffSet Location
FUNCTION VariableVAL(BYVAL VariablePTR AS DWORD,_            '  Pointer to Variable
                     BYVAL OffSet AS LONG,_                  '  Bar Offset Value
                     BYVAL VarStartAddr AS DWORD,_           '  Variable Starting Address
                     BYVAL VarSize AS LONG) EXPORT AS SINGLE '  Variable Type Size
              
   Dim ElementPTR As Single Pointer   
   
   ElementPTR = FindAddress_Var( VariablePTR, OffSet, VarStartAddr, VarSize)

   FUNCTION = @ElementPTR                                             
  
END FUNCTION   '  VariableVAL  

'  ------------------------------------
'  Returns the Value of an Easy Language Array Element at OffSet Location   
FUNCTION ArrayElementValue(BYVAL ArrayElementPTR AS DWORD,_  ' Pointer to an Array-Element
                           BYVAL OffSet AS LONG,_            ' Contains a LONG-INT Value                      
                           BYVAL StartAddr AS DWORD,_        ' Starting Address of ARRAY Series
                           BYVAL ArraySize AS DWORD )EXPORT AS SINGLE
   
   DIM ElementPTR AS Single POINTER
         
   '  Returns a Pointer to a Single Precision Value
   ElementPTR = FindAddress_Array(ArrayElementPTR, OffSet, 0, StartAddr, ArraySize)
   
   FUNCTION = @ElementPTR  ' Return the value for this Array Element
   
END FUNCTION   '  ArrayElementValue   

'  ------------------------------------
'  Finds the Price Value for the BAR-DATE at Offset Location
FUNCTION BarDateJulianVAL(BYVAL BarDatePTR AS DWORD,_
                          BYVAL OffSet AS LONG) EXPORT AS SINGLE
   DIM ElementPTR AS WORD POINTER   
   
   ElementPTR = FindAddress_Date( BarDatePTR, OffSet)

   FUNCTION = @ElementPTR   '  Returns a Julian Date Value 

END FUNCTION   '  BarDateVAL

'  ------------------------------------
'  Finds the Date Value for the BAR-DATE at Offset Location
FUNCTION BarDateYMD(BYVAL BarDatePTR AS DWORD,_
                    BYVAL OffSet AS LONG) EXPORT AS SINGLE

   FUNCTION = JulianToDate(BarDateJulianVAL(BarDatePTR, OffSet))
   
END FUNCTION   '  BarDateVAL

'  ------------------------------------
'  Finds the Minutes value for the BAR-TIME at Offset Location
FUNCTION BarTimeMinutes(BYVAL BarTimePTR AS DWORD,_
                        BYVAL OffSet AS LONG) EXPORT AS SINGLE
   DIM ElementPTR AS DWORD POINTER   
   	  
   ElementPTR = FindAddress_Time( BarTimePTR, OffSet)

   FUNCTION = @ElementPTR
        
END FUNCTION   '  BarTimeVAL
                                                                                         
'  ------------------------------------
'  Finds the TIME (HH:MM) value for the BAR-TIME at Offset Location
FUNCTION BarTimeHHMM(BYVAL BarTimePTR AS DWORD,_
                     BYVAL OffSet AS LONG) EXPORT AS SINGLE 
      
   FUNCTION = MinutesToTime(BarTimeMinutes(BarTimePTR, OffSet))

END FUNCTION   '  BarTimeHHMM(                                                                                         
                                                                                         
'  =============================================================================
'                          Testing Functions & Subs 
'  =============================================================================

'  ------------------------------------
'  Takes a number containing minutes and converts it to HHMM
FUNCTION MinutesToTime (BYVAL nMinutes AS SINGLE) EXPORT AS SINGLE   ' Does NOT check > 24-hours
   DIM Tmp AS SINGLE                     
   
   Tmp = (nMinutes / 60)
   FUNCTION = (INT(Tmp) * 100) + ((Tmp - INT(Tmp)) * 60)

END FUNCTION   '  MinutesToTime 

'  ------------------------------------
'  Takes Standard Clock HHMM Converted it the number of minutes since MidNight
FUNCTION TimeToMinutes (BYVAL ClockTime AS SINGLE) EXPORT AS SINGLE
   DIM tmpTime AS SINGLE
   DIM FractionalHour AS SINGLE
   DIM HourMinutes AS LONG
   
   IF ClockTime > 0 THEN     '  Does NOT validate Time format received
      tmpTime = (ClockTime * .01)                  
      FractionalHour = (tmpTime - INT(tmpTime))
      HourMinutes = INT(tmpTime) * 60
      FractionalHour = CINT(FractionalHour * 100)
   
      FUNCTION = HourMinutes + FractionalHour
    ELSE  
    
      FUNCTION = 0
    END IF  '  ClockTime > 0
      
END FUNCTION   '  TimeToMinutes 


'  ------------------------------------
'  Simple example of how a direct call made from EL can access an external 
'  computational process.
'
FUNCTION dllAverageClose (BYVAL ClosePtr AS DWORD,_               '  Address from &Close
                          BYVAL OffSet AS LONG,_                  '  Same as Easy Language 
                          BYVAL Length AS LONG,_                  '  Must NOT EXCEED MaxBarsBack value
                          BYVAL Scale AS SINGLE) EXPORT AS SINGLE '  Charts Current Price Scale
                          
   DIM ElementPTR AS DWORD POINTER
   DIM BarNum AS LONG
   DIM ThisCloseVal AS SINGLE
   DIM TmpVal AS SINGLE
   
   IF Length > 0 AND ClosePtr > 0 THEN
      TmpVal = 0
      
      FOR BarNum = OffSet TO OffSet + Length - 1
          ThisCloseVal =  ClosePriceVal(ClosePtr, BarNum, Scale)     '  Get this bar's CLOSE
          TmpVal = TmpVal + ThisCloseVal                             '  Sum the values  
      NEXT BarNum

      FUNCTION = TmpVal / Length                                     '  Create an Average
   ELSE
      FUNCTION = 0
   END IF   '  Length > 0 AND ClosePtr > 0
   
END FUNCTION   '  dllAverageClose 

'  ------------------------------------
'  Simple example of how a direct call can be made from EL
'  Passing the actual price value, instead using the address,
'  of the price object to compute a new value
'
FUNCTION DoubleIt (BYVAL AnyPrice AS SINGLE) EXPORT AS SINGLE

   FUNCTION = AnyPrice * 2                                           ' Double the Price
END FUNCTION   '  DoubleIt 
                                
'  ------------------------------------
'  Simple example of how a direct call can be made from EL
'  Passing the actual value of a user variable instead using the 
'  address the variable object to change the value of the variable
'
FUNCTION IncrNum (BYVAL AnyVariableNum AS SINGLE) EXPORT AS SINGLE   ' Increments a Number by 1

   FUNCTION = AnyVariableNum + 1
END FUNCTION   '  IncrNum                     

'  ------------------------------------
'  Pass an ELDate-Value as a String and Convert it into a US Calendar
'  date in STRING format.  The EL date-value is then passed back as a 
'  STRING to show how STRINGS can be passed back and forth between 
'  the DLL and Easy Language. 
'
FUNCTION MakeCalDate (ELDateStr AS ASCIIZ, _'  Pass Date as Fixed STRING
                      BYVAL NotUS AS LONG) _           '  ZERO or NOT-ZERO
                      EXPORT AS STRING                '  Return a Modified STRING   
  
   DIM ELDateVal AS LONG               
   DIM TmpDate AS STRING
   DIM Yr$, Mnth$, Day$

   
   'ELDateVal = VAL(@ELDateStr)         '  Make it a Numeric 
   ELDateVal = Val(ELDateStr)         '  Make it a Numeric 
   ELDateVal = 19000000 + ELDateVal    '  Add Full Century Value
   TmpDate = TRIM$(STR$(ELDateVal))    '  Make it a STRING & Remove SPACE     
   
   Yr$ = LEFT$(TmpDate$, 4)            '  Capture Year with Century
   Mnth$ = MID$(TmpDate, 5, 2)         '  Grab the Month Value
   Day$ = RIGHT$(TmpDate$, 2)          '  Day number goes here
   
   IF ISTRUE(NotUS) THEN               '  If NOT in US,...
                                                       '                  dd mm yyyy
      FUNCTION = Day$ + "-" + Mnth$ + "-" + Yr$ + $NUL ' Give this format 21-02-2003
   ELSE
                                                       '                  mm dd yyyy
      FUNCTION = Mnth$ + "-" + Day$ + "-" + Yr$ + $NUL ' US is this format 02-21-2003
   END IF   '  ISTRUE(NotUS) 

END FUNCTION   '  MakeCalDate          



'  =============================================================================
'                                END of DLL SOURCE CODE
'  =============================================================================
